const path = require('path');
const { URL } = require('url');
const glob = require('glob');
const allSettled = require('promise.allsettled');
const { injectScriptWithHash, computeDpcHash } = require('./inject-script');
const { removeHeaders } = require('./remove-headers');

// adding env var fallbacks allows us to build locally with the Netlify CLI
const {
  DEPLOY_PRIME_URL = 'https://deploy-preview-2--dpcbp.netlify.app',
} = process.env;

module.exports = {
  async onPostBuild({ constants: { CONFIG_PATH, PUBLISH_DIR }, utils }) {
    // only enable deploy preview commenting for deploy previews
    if (process.env.CONTEXT !== 'deploy-preview') {
      return;
    }

    const dpcHash = computeDpcHash();
    if (!dpcHash) {
      return utils.build.failPlugin("Deploy preview commenting is currently only supported for GitHub repos");
    }

    // remove iframe security headers so DPC works
    const headersFilePath = path.join(PUBLISH_DIR, '_headers');
    const tomlFilePath = CONFIG_PATH;

    removeHeaders(headersFilePath);
    removeHeaders(tomlFilePath);

    // add the DPC script to all HTML files
    const files = glob.sync(`${PUBLISH_DIR}/**/*.html`);
    const promises = files.map(injectScriptWithHash(dpcHash));

    const results = await allSettled(promises);
    const successes = results.filter(({ status }) => status === 'fulfilled')
      .length;
    const failures = results.filter(({ status }) => status === 'rejected');

    console.log(`injected the DPC script into ${successes} pages`);

    if (failures.length > 0) {
      console.error(
        'Unable to add the Deploy Preview Commenting script to the following pages:',
      );
      console.error(failures.join('\n'));
    }

    const url = new URL(DEPLOY_PRIME_URL);
    const [subdomain] = url.hostname.split('.');
    const dpcURL = `https://netlify-dpc.netlify.app/${subdomain}/`;

    utils.status.show({
      title: 'Deploy Preview Commenting enabled!',
      summary: `View and discuss at ${dpcURL}`,
    });
  },
};
