const fs = require('fs');
const parseGitHub = require('parse-github-repo-url');
const unified = require('unified');
const parse = require('rehype-parse');
const visit = require('unist-util-visit');
const h = require('hastscript');
const stringify = require('rehype-stringify');

// adding env var fallbacks allows us to build locally with the Netlify CLI
const {
  REPOSITORY_URL = 'https://github.com/netlify/netlify-plugin-deploy-preview-commenting',
  DEPLOY_PRIME_URL = 'https://deploy-preview-2--dpcbp.netlify.app',
  REVIEW_ID = 2,
} = process.env;


const addScriptNode = ({hash}) => (tree) => {
  visit(tree, 'element', (node) => {
    if (node.tagName === 'body') {
      node.children.push(h('script', `window.___NETLIFY_DPC___="${hash}"`));
      node.children.push(
        h('script', { src: 'https://dpcbp.netlify.app/netlify-dpc.js' }),
      );
    }
  });
};

exports.computeDpcHash = () => {
  const ghInfo = parseGitHub(REPOSITORY_URL);
  if(!ghInfo) {
    return;
  }

  const [owner, repo] = ghInfo

  const info = {
    netlify_prime_url: DEPLOY_PRIME_URL,
    pull_request_id: REVIEW_ID,
    owner,
    repo,
  };

  return Buffer.from(JSON.stringify(info)).toString('base64');
}

exports.injectScriptWithHash = (hash) => (filePath) =>
  new Promise((resolve, reject) => {
    try {
      const contents = fs.readFileSync(filePath, 'utf-8');

      const markup = unified()
        .use(parse)
        .use(addScriptNode, {hash})
        .use(stringify)
        .processSync(contents)
        .toString();

      fs.writeFileSync(filePath, markup);

      resolve(filePath);
    } catch (error) {
      console.error(`error injecting script into ${filePath}`);
      console.error(error.message);

      reject(filePath);
    }
  });
